@@ -16,13 +16,27 @@ VERSION=`head -n 1 debian/changelog | cut -f2 -d\( | cut -f1 -d')' | cut -f1 -d-
 # CVS repositories..
 PACKAGE_CVSROOT=catalyst/codeforge/php/${PACKAGE}
 
-echo -n "refresh ChangeLog? (y/n) [n]: "
+echo -n "refresh ChangeLog? [n]: "
 read ANS
 if [ "$ANS" = "y" -o "$ANS" = "Y" ]
 then
   cvs2cl >/dev/null 2>&1
 fi
 
+# Ensure we have latest Luceneserver locally..
+if [ -d luceneserver/lib ] ; then
+  echo -n "refresh luceneserver libs and code? [n] :"
+  read ANS
+  if [ "$ANS" = "y" ] ; then
+    rm -rf luceneserver
+  fi
+fi
+if [ ! -d luceneserver/lib ]; then
+  rm -rf luceneserver
+  echo "exporting a copy of Luceneserver.."
+  cvs export -r HEAD -d luceneserver catalyst/luceneserver
+fi
+
 echo "constructing package to build.."
 HOMEDIR=`pwd`
 cd ..
@@ -59,13 +73,6 @@ find . -iname "*.db" -exec chmod 664 {} \;
 find . -iname ".htaccess" -exec chmod 664 {} \;
 find . -iname "ChangeLog" -exec chmod 664 {} \;
 
-# Ensure we have latest Luceneserver locally..
-if [ ! -d luceneserver/lib ]; then
-  rm -rf luceneserver
-  echo "exporting a copy of Luceneserver.."
-  cvs export -r HEAD -d luceneserver catalyst/luceneserver
-fi
-
 # Build it!
 dpkg-buildpackage -tc -rsudo -sgpg -m"${USER}" -k"${USER}"
 
