@@ -351,12 +351,12 @@ public class Application implements IPackage, Constants
     return oldValue;
   }
 
-  public String getProperty(String key)
+  public synchronized String getProperty(String key)
   {
     return properties.getProperty(key);
   }
 
-  public Properties getProperties()
+  public synchronized Properties getProperties()
   {
     return properties;
   }
@@ -366,7 +366,7 @@ public class Application implements IPackage, Constants
    * isn't even in the cache yet.
    */
 
-  private void load(File file) throws IOException
+  private synchronized void load(File file) throws IOException
   {
     Log.log(ELog.INFO, "Reading Application properties from: " + file.getPath());
     InputStream in = new BufferedInputStream(new FileInputStream(file));
@@ -624,7 +624,7 @@ public class Application implements IPackage, Constants
    * files, and write out any Applications that have changed.
    */
 
-  private static void scanDirectory()
+  private static synchronized void scanDirectory()
   {
     // Log.debug("Scanning for changed Application properties files");
     File directory = new File(System.getProperty(DIRECTORY_PROPERTY).trim());
