@@ -21,7 +21,6 @@ package nz.net.catalyst.lucene.server;
 import java.io.File;
 import java.io.IOException;
 import java.util.Arrays;
-import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.LinkedList;
@@ -40,6 +39,8 @@ import org.apache.lucene.search.Hits;
 import org.apache.lucene.search.RangeQuery;
 import org.apache.lucene.search.Searcher;
 import org.apache.lucene.search.TermQuery;
+import org.apache.lucene.search.SortField;
+import org.apache.lucene.search.Sort;
 
 /**
  * Execute a Lucene Query.
@@ -111,7 +112,6 @@ public class Query implements IPackage, Constants
     
     //hu.origo.lucenetools.iac.IndexAccessControl iac;
   	long beforeSearch;
-  	long beforeSort;
   	long beforeOutput;
     try {
       exceptionMessage = "closing idle IndexWriter";
@@ -135,14 +135,26 @@ public class Query implements IPackage, Constants
       		String message = "Error during query: Index not found. Index documents first or check settings.";
     	  	Log.error(message);
 	      	return error(message);
-      } 
-      beforeSearch = System.currentTimeMillis();      
-      exceptionMessage = "searching Index";
-      
-      Hits hits = searcher.search(query);
-      int hitCount = hits.length();
+      }
 
-	  Log.debug("Found " + hitCount + " results in " + (System.currentTimeMillis() - beforeSearch) + "ms");
+      // Assemble any sort fields specified for this search..
+      SortField[] sortfields = getSortFields();     
+     
+      // Execute either a sorted or unsorted query..
+      exceptionMessage = "searching Index";
+      beforeSearch = System.currentTimeMillis();
+      Hits hits = null;
+      int hitCount = 0;
+      if (sortfields != null) {
+        hits = searcher.search(query, new Sort(sortfields));
+        hitCount = hits.length();
+      }
+      else {
+        hits = searcher.search(query);
+        hitCount = hits.length();
+      }
+
+      Log.debug("Found " + hitCount + " results in " + (System.currentTimeMillis() - beforeSearch) + "ms");
 
       // Find out how many results to return.  Default is all of them
       // need to do this now to ensure the Sort-Limit values make sense.
@@ -162,151 +174,8 @@ public class Query implements IPackage, Constants
     	} catch (Exception e) { /* Ignore */ }
     	--first;
 
-      beforeSort = System.currentTimeMillis();	  
 
-    	//SoftReference[] docs = null; //no longer using softreferences
-    	SortObject[] docs = null; //USE HARD REFERENCES NOW
-    	Sort[] sortList = getSortList();
-
-      if (sortList != null) {
-		//calculate sort limits.
-      	int sortLimitSystem = -1;
-      	int sortLimitUser   = -1;
-      	try {
-      	  sortLimitSystem = Integer.parseInt(System.getProperty(PACKAGE + "SortLimitSystem"));
-      	  sortLimitUser   = Integer.parseInt(input.get("Sort-Limit"));
-      	} catch (NumberFormatException e) { /* ignore */ }
-
-      	Log.debug("sortLimitSystem: " + sortLimitSystem);
-      	Log.debug("sortLimitUser: " + sortLimitUser);
-
-		//validate limits.
-      	if (sortLimitSystem != -1 && sortLimitUser > sortLimitSystem) {
-      	  sortLimitUser = sortLimitSystem;
-      	  Log.debug("Resetting Sort-Limit sent from client because was greater than the system sort limit. Sort-Limit=" + sortLimitUser + " Sort-Limit=" + sortLimitUser);
-      	}
-      	if (sortLimitUser != -1 && limit > sortLimitUser) {
-      	  sortLimitUser = -1;
-      	  Log.debug("Ignoring Sort-Limit sent from client because it is less than their specified limit. Limit=" + limit + " Sort-Limit=" + sortLimitUser);
-      	}
-		//finished calculating sort limits
-
-      	//check the number of results against the sortLimitXXXXs to see if we should sort.
-      	if (sortLimitUser != -1 && hitCount > sortLimitUser) {
-      		Log.debug("Number of results exceeds user sort limit. HitCount=" + hitCount + " Sort-Limit=" + sortLimitUser);
-      		return error("Number of results exceeds user sort limit. Please restrict search or do not sort results.");
-      	}
-      	if (sortLimitSystem != -1 && hitCount > sortLimitSystem) {
-      		Log.debug("Number of results exceeds system sort limit. HitCount=" + hitCount + " System Sort-Limit=" + sortLimitSystem);
-      		return error("Number of results exceeds system sort limit. Please restrict search or do not sort results.");
-      	}
-      		
-      	//there is some sorting criteria.
-        if (Log.isDebugEnabled()) {
-	          //output sort criteria
-	          Log.debug("Sort override requested -- will retrieve all matching documents into memory for sorting.");
-	          StringBuffer sb = new StringBuffer();
-	          for (int i = 0; i < sortList.length; ++i) {
-		            if (i > 0) sb.append(", ");
-		            sb.append(sortList[i].field == null ? RANK : sortList[i].field);
-		            sb.append(':');
-		            sb.append(sortList[i].descending ? "Desc" : "Asc");
-	          }
-	          Log.debug("Sort order is: " + sb);
-        }
-
-        // Use Soft-References to hold the sort data.  If we run out
-        // of memory, then some of them will "evaporate".  In that
-        // case, the sort routime will notice and throw an
-        // "Out-of-Memory" message back to the client.
-		// we have stopped using soft-references now as sorting was failing
-		// because the softreferences were claimed before increasing the 
-		// JVM heap which wasn't good (TM).
-        //docs = new SoftReference[hitCount];
-      	docs = new SortObject[hitCount]; //USE HARD REFERENCES NOW
-
-        exceptionMessage = "building sort keys";
-
-									/*
-									//BEGIN FILTER-SORT
-											if (hitCount > limit) {
-												Log.debug("about to try filter-sort approach. hitcount="+hitCount+" limit="+limit);
-												java.util.Vector filteredHits = new java.util.Vector(limit);
-												
-												Comparator comparator1 = new DocCompare(sortList);
-												
-												for (int i = 0; i < limit; i++) {
-												  SortObject item = makeSortObject(sortList, hits, i);
-												  filteredHits.add(item);
-												}
-									
-												java.util.Collections.sort(filteredHits, comparator1);
-												Log.debug("initial sort completed. filteredHits.size() = " + filteredHits.size());
-									      		SortObject limitItem = (SortObject) filteredHits.get(limit-1);
-									      		
-										        for (int i = limit; i < hitCount; i++) {
-										          //docs[i] = new SoftReference(makeSortObject(sortList, hits, i));
-										      	  
-										      	  SortObject item = makeSortObject(sortList, hits, i);
-										          int result = comparator1.compare(item, limitItem);
-										          //Log.debug(String.valueOf(result));
-										      	  if ( result > 0) {
-										      	  	 //Log.debug("Just added item to the list");
-										      	  	 filteredHits.add(item);
-										      	  }
-										      	  //leave this out for now.
-										      	  //java.util.Collections.sort(filteredHits, comparator1);	 
-										        }
-												Log.debug("Full sort completed. filteredHits.size() = " + filteredHits.size());
-												
-												SortObject[] a = new SortObject[1];
-												
-										        docs = (SortObject[]) filteredHits.toArray(a);
-										        
-										        Log.debug("docs.length="+docs.length);
-									        		
-										    } //end of if hitCount > limit
-										    else {
-										    	Log.debug("using standard sort approach (i.e. no filter)");
-										    	for (int i = 0; i < hitCount; i++)
-										    	  //docs[i] = new SoftReference(makeSortObject(sortList, hits, i));
-									  	    	  docs[i] = makeSortObject(sortList, hits, i);
-										    }        
-									//END FILTER-SORT        
-									*/
-
-		long beforeResultsRetrieve = System.currentTimeMillis();
-		
-		Log.debug("about to put result set into array for sorting");
-      	for (int i = 0; i < hitCount; i++) {
-      	  //docs[i] = new SoftReference(makeSortObject(sortList, hits, i)); 
-      	  docs[i] = makeSortObject(sortList, hits, i); //USE HARD REFERENCES NOW
-      	  
-      	}
-      	
-      	long afterResultsRetrieve = System.currentTimeMillis();
-      	
-      	Log.debug("Time taken to retrieve results into docs array: " + (afterResultsRetrieve - beforeResultsRetrieve) + "ms.");
-        
-        Comparator comparator = new DocCompare(sortList);
-     	
-        try {
-          exceptionMessage = "sorting results";
-          Arrays.sort(docs, comparator);
-        } catch (QueryException e) {
-          return error("Ran out of memory while sorting " + hitCount +
-                       " search results.  Please restrict search or do not sort them.");
-        } catch (OutOfMemoryError e) {
-        	Log.error("OutOfMemoryError - while sorting " + hitCount + " documents.");
-        	return error("Ran out of memory while sorting " + hitCount +
-        				 " search results.  Please restrict search or do not sort them.");
-        }
-        
-      	long afterSort = System.currentTimeMillis();
-      	Log.debug("Sort took (in ms):" + (afterSort - beforeSort));
-      }
-
-	 beforeOutput = System.currentTimeMillis();
+    	beforeOutput = System.currentTimeMillis();
 
       exceptionMessage = "retrieving results";
       int last = first + limit;
@@ -315,16 +184,6 @@ public class Query implements IPackage, Constants
       for (int i = first; i != last && i < hitCount; ++i) {
         int index = i;
 
-        if (docs != null) {
-          //SortObject sort = (SortObject)docs[i].get(); //no longer using soft references.
-          SortObject sort = (SortObject)docs[i]; //USE HARD REFERENCES NOW
-          if (sort == null)
-            return error("Ran out of memory while retrieving sorted " +
-                         "documents after sorting " + hitCount +
-                         " search results.  Please restrict search " +
-                         "or do not sort them.");
-            index = sort.index;  //seems redundant (unreachable)
-        }
         Document d = hits.doc(index);
         float score = hits.score(index);
         response.add(I, String.valueOf(i+1)); 
@@ -349,13 +208,6 @@ public class Query implements IPackage, Constants
         }
       }
 
-      // for (int i = 0 ; i < hits.length() && i < 10; i++)
-      // {
-      //   Document d = hits.doc(i);
-      //   Log.debug("Found " + DOMAIN + '=' + d.get(DOMAIN) + ", " +
-      //                      ID + '=' + d.get(ID));
-      // }
-
     } catch (IOException e) {
       String message = "Error during query: " + e.toString();
       Log.error(message);
@@ -372,14 +224,13 @@ public class Query implements IPackage, Constants
       }
     }
 
-	 long end = System.currentTimeMillis();
+	  long end = System.currentTimeMillis();
 	 
-  	 Log.debug("ReadQuery:" + String.valueOf(beforeParse - queryStart));
-  	 Log.debug("Parse/BuildQuery:" +  String.valueOf(beforeOpenSearcher - beforeParse));
-  	 Log.debug("OpenSearcher:" + String.valueOf(beforeSearch - beforeOpenSearcher));
-  	 Log.debug("Search: " + String.valueOf(beforeSort - beforeSearch));
-  	 Log.debug("Sort: " +   String.valueOf(beforeOutput - beforeSort));
-  	 Log.debug("Output: " +String.valueOf(end - beforeOutput));
+  	Log.debug("ReadQuery:" + String.valueOf(beforeParse - queryStart));
+  	Log.debug("Parse/BuildQuery:" +  String.valueOf(beforeOpenSearcher - beforeParse));
+  	Log.debug("OpenSearcher:" + String.valueOf(beforeSearch - beforeOpenSearcher));
+  	Log.debug("Search: " + String.valueOf(beforeOutput - beforeSearch));
+  	Log.debug("Output: " +String.valueOf(end - beforeOutput));
 
     return response;
   }
@@ -575,56 +426,51 @@ public class Query implements IPackage, Constants
    * default descending RANK sort.
    */
 
-  private Sort[] getSortList()
+  private SortField[] getSortFields()
   {
     String[] fieldSpecs = input.get(SORT, USE_APP, SPLIT);
     if (fieldSpecs.length == 0)
       return null;
     
-    Sort[] result = new Sort[fieldSpecs.length];
+    SortField[] result = new SortField[fieldSpecs.length];
 
     for (int i = 0; i < result.length; ++i)
     {
       String[] subSpec = Util.split(fieldSpecs[i], ":");
 
       if (subSpec.length < 1 || subSpec.length > 2)
-        throw new QueryException("Invalid sort spec: \"" + input.get(SORT) + '"');
+        throw new QueryException("Invalid sort spec: \"" + input.get(SORT) + "\"");
 
       String field = subSpec[0];
+      Log.debug("Sorting by " + field + ".");
 
       if (field.equals(RANK))
-        field = null;
-
-      boolean descending = false;
-
-      if (subSpec.length == 2)
       {
-        String direction = subSpec[1].toLowerCase();
-        if (direction.startsWith("d"))
-          Log.debug("Setting sort order for field(" + field + ") to descending.");
-          descending = true;
+          result[i] = new SortField(null, SortField.SCORE);
+      } else {
+        boolean descending = false;
+    
+        if (subSpec.length == 2)
+        {
+          String direction = subSpec[1].toLowerCase();
+          if (direction.startsWith("d"))
+            Log.debug("Setting sort order for field(" + field + ") to descending.");
+            descending = true;
+        }
+        result[i] = new SortField(field, descending);
       }
-      result[i] = new Sort(field, descending);
     }
 
-    // See if we're doing a descending RANK sort (the default)
-
-//    for (int i = 0; i < result.length; ++i)
-//    {
-//      if (result[i].field != null || !result[i].descending)
-//        return result;
-//    }
-//    return null;
-	//rewrote this to try and make it clearer.
-	if (result.length == 1 && result[0].field == null && result[0].descending == false) {
-		Log.debug("Sort term specified was default term.  Ignoring sort request.");
-		return null;
-	} else {
-		return result;
-	}
+    // Check if we're doing a descending simple RANK sort (the default)
+    if (result.length == 1 && result[0].getType() == SortField.SCORE && result[0].getReverse() == false) {
+        Log.debug("Sort term specified was the default ordering.  Skipping sort request.");
+        return null;
+    } else {
+        return result;
+    }
 
   }
-
+  
   /**
    * Build an error response for sending back to the client.
    *
@@ -640,104 +486,5 @@ public class Query implements IPackage, Constants
     return response;
   }
 
-  /**
-   * Build an object containing all the sort keys for one document.
-   * This will be sorted using a custom Comparator
-   */
-
-  private SortObject makeSortObject(Sort[] sortList, Hits hits, int index)
-    throws IOException
-  {
-    SortObject result = new SortObject(index, sortList.length);
-    Document d = hits.doc(index);
-
-    for (int i = 0; i < sortList.length; ++i)
-    {
-      if (sortList[i].field == null)
-        result.key[i] = new Float(hits.score(index));
-      else
-      {
-        String key = d.get(sortList[i].field);
-        if (key == null)
-          key = "";
-        result.key[i] = key;
-        
-        //Log.debug("Key = " + key);
-        
-      }
-    }
-    return result;
-  }
-
-  private static class DocCompare implements Comparator
-  {
-    Sort[] sortList;  // The key to the correct ordering.
-
-    DocCompare(Sort[] sortList)
-    {
-      this.sortList = sortList;
-    }
-
-    // We're comparing objects stored within a soft-reference.  If any
-    // of them have evaporated, then that means we've run out of
-    // memory.
-
-    public int compare(Object objA, Object objB)
-    {
-      //SortObject a = (SortObject)((SoftReference)objA).get();
-      //SortObject b = (SortObject)((SoftReference)objB).get();
-      SortObject a = (SortObject) objA; //USE HARD REFERENCES NOW
-	  SortObject b = (SortObject) objB; //USE HARD REFERENCES NOW
-    		  
-      if (a == null || b == null)
-        throw new QueryException("Not enough memory to hold all results");
-
-      for (int i = 0; i < a.key.length; ++i)
-      {
-        int result = a.key[i].compareTo(b.key[i]);
-        if (result != 0)
-        {
-          if (sortList[i].descending)
-            result = -result;
-
-          return result;
-        }
-      }
-      return 0;
-    }
-  }
-
-  /**
-   * This object contains the sort keys for one document, plus a
-   * reference to the document.  Once all objects have been
-   * appropriately sorted, then they can be refetched and returned to
-   * the client.
-   */
-  private static class SortObject
-  {
-    final int index;
-    final Comparable[] key;
-
-    SortObject(int index, int keyCount)
-    {
-      this.index = index;
-      this.key = new Comparable[keyCount];
-    }
-  }
-
-  private static class Sort
-  {
-    // The field name to sort on.  Null signifies the RANK
-    final String field;
-
-    // Sort direction
-    final boolean descending;
-
-    Sort(String field, boolean descending)
-    {
-      this.field = field;
-      this.descending = descending;
-    }
-  }
 }
 
